// Select the needed modules
import { 
  select, 
  csv, 
  scaleLinear, 
  max, 
  scaleBand, 
  axisLeft,
  axisBottom,
  format
} from 'd3';


// Capture in a variable the DOM component 'svg'
const svg = select('svg');

// Capture from the attributes 'height' and 'width' from the 
// DOM component 'svg'
// Use (+) to parse string type to int type.
const height = +svg.attr('height');
const width = +svg.attr('width');



// Create 'promise' lamda with rendering instructions
// pass as parameter 'data' array, captured from using 'csv'
const render = data => {
  
  // Split data for X axis and Y axis
  const xValue = d => d.population
  const yValue = d => d.country
  
  // Set Margins following D3 guidelines
  const margin = {top: 50, right: 40, bottom: 60, left: 150}
  
  // Calculating width and height form the chart 'canvas'
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;
  
  // Define X scale as linear scale and 
  // defining the dataset 'domain' in this case the population field
  // in X axis. Defining the 'range' to which the values will
  // extend, the whole available canvas width 'innerWidth'
  const xScale = scaleLinear()
  	.domain([0, max(data,xValue)])
    .range([0, innerWidth]);    
  
  // Define Y scale
  // defining the dataset 'domain', in this case the countries field
  // in the Y axis. Defininf the range to whcih the values will
  // extend, the whole available canvas height 'innerHeight'
  const yScale = scaleBand()
  	.domain(data.map(yValue))
    .range([0,innerHeight])
  	.padding(0.1);  
  
  // Crate a group and render all the chart components inside
  // User transform function to translate (line up) chart componets
  // to the top/left marging
  const g = svg.append('g')
  	.attr('transform',`translate(${margin.left},${margin.top})`)
  
  // Format xAxis tick, replace 'G' by 'B' using lambda iteration 
  // thru each member of the axis
  const xAxisTickFormat = number => format('.3s')(number).replace('G','B')
  
  // Get the buttom axis, formates and define the size
  const xAxis = axisBottom(xScale)
  	.tickFormat(xAxisTickFormat)
  	.tickSize(-innerHeight);
  
  // Remove tick and lines from the Y scale axis
  // using DOM class references to '.domain' and '.tick lines'
  g.append('g')
    	.call(axisLeft(yScale))
  		.selectAll('.domain, .tick line')
  		.remove();
  
  // Adding the xAxis to a group, that way I can remove the domain line
  const xAxisG = g.append('g').call(xAxis)
  	.attr('transform',`translate(0,${innerHeight})`);
  
  // Remove the domain axis
  xAxisG.selectAll('.domain').remove();
  
  
  // Add title to X axis, assigning class attribute to use a reference in CSS
  xAxisG.append('text')
  		.attr('class', 'axis-label')
  		.attr('y', 45)
  		.attr('x', innerWidth / 2)
  		.attr('fill', 'black')
  		.text('Population');
  
  // Add bars and passing axis and data values promises as parameters
  g.selectAll('rect').data(data)
  	.enter().append('rect')
  		.attr('y', d => yScale(yValue(d)))
  		.attr('width', d => xScale(xValue(d)))
  	  .attr('height',yScale.bandwidth());  
  
  // Add graph title
  g.append('text')
  	.attr('class', 'title')
  	.attr('y', -10)
  	.text('Top 10 Most Populous Countries');
};

csv('data.csv').then(data => {
 data.forEach(d => {
   d.population = +d.population * 1000;
 }); 
 render(data);
});